#include <windows.h>
#include "resource.h"
#include "SoundRecorder.h"

#define IDM_ABOUT 20001

#define TIMER_ID 666

LPCTSTR RECORDING_TEXT = "Recording";
LPCTSTR PAUSED_TEXT = "Paused";
LPCTSTR STOPPED_TEXT = "Stopped";

HINSTANCE ghInst;

HICON mainIcon;
HICON recordIcon;
HICON pauseIcon;
HICON stopIcon;
HICON volIcon;
HBITMAP screenBmp;

HRECORDER hRec;

HFONT screenFont;
HDC memDc;

//procedure de dialogue d'authentification
INT_PTR WINAPI RecordDialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

//procedure de dialogue  propos
INT_PTR WINAPI AboutDialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

void OnInitDialog(HWND hDlg);
void OnSysCommand(HWND hDlg, WPARAM id);
void OnPaint(HWND hDlg);
void OnRecord(HWND hDlg);
void OnPause(HWND hDlg);
void OnStop(HWND hDlg);
void OnVolume();
void OnDestroy();

void UpdateScreen(HWND hDlg);

//afficher les erreurs
void DisplayLastError();

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow) 
{
	ghInst = hInstance;

	mainIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDR_MAINFRAME));
	recordIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_RECORD));
	pauseIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_PAUSE));
	stopIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_STOP));
	volIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_VOLUME));
	screenBmp = LoadBitmap(hInstance, MAKEINTRESOURCE(IDB_SCREEN));

	screenFont = CreateFont(
		-24,
		0, 0, 0, FW_REGULAR, 0, 0, 0,        
		DEFAULT_CHARSET,
		OUT_TT_ONLY_PRECIS,
		CLIP_DEFAULT_PRECIS,
		ANTIALIASED_QUALITY,
		VARIABLE_PITCH|FF_SWISS,
		"Arial"
	);

	hRec = SoundRecorderInit();
	if (hRec == NULL)
	{
		DisplayLastError();
		return -1;
	}

	DialogBox(hInstance, 
				MAKEINTRESOURCE(IDD_RECORDERDIALOG), 
				NULL, 
				(DLGPROC)RecordDialogProc);

	return 0;
}

INT_PTR WINAPI RecordDialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
    {
    case WM_INITDIALOG:
		OnInitDialog(hDlg);
		return FALSE;

	case WM_SYSCOMMAND:
		OnSysCommand(hDlg, wParam);
        return 0;

	case WM_PAINT:
		OnPaint(hDlg);
		break;

    case WM_COMMAND:
        switch (LOWORD(wParam))
        {
        case IDC_RECORD:
            OnRecord(hDlg);
            break;

		case IDC_PAUSE:
			OnPause(hDlg);
            break;

		case IDC_STOP:
			OnStop(hDlg);
			break;

		case IDC_VOLUME:
			OnVolume();
			break;
        }
		UpdateScreen(hDlg);
        return TRUE;

	case WM_TIMER:
		UpdateScreen(hDlg);
		return 0;
    
	case WM_DESTROY:
		OnDestroy();
        return TRUE;
    }
    return FALSE;    
}

INT_PTR WINAPI AboutDialogProc( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
    case WM_COMMAND:
        switch (LOWORD(wParam))
        {
        case IDOK:
        case IDCANCEL:
            EndDialog(hDlg, LOWORD(wParam));
            return TRUE;
        }
        break;
    }
    return FALSE;
}

void DisplayLastError()
{
	LPTSTR lpMsgBuf;

	FormatMessage(
				FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
				NULL,
				GetLastError(),
				MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
				(LPTSTR) &lpMsgBuf,
				0,
				NULL
				);

	MessageBox(NULL, lpMsgBuf, "Error", MB_ICONERROR );
	LocalFree( lpMsgBuf );
}

void OnInitDialog(HWND hDlg)
{
	HMENU sysmen;

	//ajout about command
	sysmen = GetSystemMenu(hDlg, FALSE);
	AppendMenu(sysmen, MF_SEPARATOR, 0, NULL);
	AppendMenu(sysmen, MF_ENABLED, IDM_ABOUT, "&About...");

	//init icones
	SendMessage(hDlg, WM_SETICON, ICON_BIG, (LPARAM)mainIcon);
	SendMessage(GetDlgItem(hDlg, IDC_RECORD), BM_SETIMAGE , IMAGE_ICON, (LPARAM)recordIcon);
	SendMessage(GetDlgItem(hDlg, IDC_PAUSE), BM_SETIMAGE , IMAGE_ICON, (LPARAM)pauseIcon);
	SendMessage(GetDlgItem(hDlg, IDC_STOP), BM_SETIMAGE , IMAGE_ICON, (LPARAM)stopIcon);
	SendMessage(GetDlgItem(hDlg, IDC_VOLUME), BM_SETIMAGE , IMAGE_ICON, (LPARAM)volIcon);

	memDc = CreateCompatibleDC(GetDC(hDlg));
    DeleteObject(SelectObject(memDc, screenBmp));
}

void OnSysCommand(HWND hDlg, WPARAM id)
{
	switch (id)
    {
	case IDM_ABOUT:
		DialogBox(ghInst, MAKEINTRESOURCE(IDD_ABOUTDIALOG), hDlg, (DLGPROC)AboutDialogProc);
		break;
	case SC_CLOSE:
		EndDialog(hDlg, LOWORD(id));
        break;
	}
}

void OnRecord(HWND hDlg)
{
	SYSTEMTIME currTime;
	TCHAR filename[MAX_PATH*2];
    
	CheckDlgButton(hDlg, IDC_RECORD, BST_CHECKED);

	if (SoundRecorderGetState(hRec) == RS_STOPPED)
	{
		GetLocalTime(&currTime);

		wsprintf(filename, "%d%02d%02d_%02d%02d%02d.mp3",
			currTime.wYear,
			currTime.wMonth,
			currTime.wDay,
			currTime.wHour,
			currTime.wMinute,
			currTime.wSecond);
		
		SoundRecorderSetOutputFile(hRec, filename);

		SoundRecorderStart(hRec);
		SetTimer(hDlg, TIMER_ID, 1000, NULL);
	}
}

void OnPause(HWND hDlg)
{
	if (SoundRecorderGetState(hRec) == RS_PAUSED)
	{
		SoundRecorderResume(hRec);
		SetTimer(hDlg, TIMER_ID, 1000, NULL);
	}
	else if (SoundRecorderGetState(hRec) == RS_RECORDING)
	{
		SoundRecorderPause(hRec);
		KillTimer(hDlg, TIMER_ID);
	}
		
	CheckDlgButton(hDlg, IDC_PAUSE,
		(SoundRecorderGetState(hRec) == RS_PAUSED) ? BST_CHECKED:BST_UNCHECKED);
}

void OnStop(HWND hDlg)
{
	SoundRecorderStop(hRec);
	CheckDlgButton(hDlg, IDC_RECORD, BST_UNCHECKED);
	CheckDlgButton(hDlg, IDC_PAUSE, BST_UNCHECKED);
	KillTimer(hDlg, TIMER_ID);
}

void OnVolume()
{
	STARTUPINFO inf;
    PROCESS_INFORMATION pinf;

	inf.cb = sizeof(STARTUPINFO);
	inf.lpReserved = NULL;
	inf.lpDesktop = NULL;
	inf.lpTitle = NULL;
	inf.dwFlags = 0;
	inf.cbReserved2 = 0;
	inf.lpReserved2 = NULL;
	if (!CreateProcess(NULL, "SndVol32.exe -R -D 0", NULL, NULL, FALSE,
		CREATE_NEW_CONSOLE, NULL, NULL, &inf, &pinf))
		DisplayLastError();
}

void OnDestroy()
{
	SoundRecorderClose(hRec);
}

void OnPaint(HWND hDlg)
{
	HWND hScreen;
	HDC dc;
	PAINTSTRUCT ps;
	RECT rct;
	LPCTSTR str;
	TCHAR hour[50];
	int nrec, nsecs, nmins;

	hScreen = GetDlgItem(hDlg, IDC_SCREEN);
	if (GetUpdateRect(hScreen, &rct, FALSE) != 0)
    {
		dc = BeginPaint(hScreen, &ps);		
		
		GetClientRect(hScreen, &rct);
		BitBlt(dc, 0, 0, rct.right, rct.bottom, 
			memDc, rct.left, rct.top, SRCCOPY);
		SetBkMode(dc, TRANSPARENT);
		SetTextColor(dc, RGB(255, 255, 210));
		DeleteObject(SelectObject(dc, screenFont));
		switch (SoundRecorderGetState(hRec))
		{
		case RS_RECORDING: str = RECORDING_TEXT; break;
		case RS_PAUSED: str = PAUSED_TEXT; break;
		case RS_STOPPED: str = STOPPED_TEXT; break;
		}
		TextOut(dc, 20, 15, str, lstrlen(str));

		nrec = SoundRecoderGetSecondsRecorded(hRec);
		nsecs = nrec % 60;
		nmins = (nrec / 60) % 60;
		wsprintf(hour, "%02d:%02d", nmins, nsecs);

		TextOut(dc, 20, 48, hour, lstrlen(hour));

		EndPaint(hScreen, &ps);
	}
}

void UpdateScreen(HWND hDlg)
{
	RECT rect;
	GetClientRect(hDlg, &rect);
	InvalidateRect(hDlg, &rect, FALSE);
	UpdateWindow(hDlg);
}
