#include "SoundRecorder.h"
#include <stdio.h>

void printlog(const char * fmt, va_list args)
{
}

HRECORDER SoundRecorderInit()
{
	MMRESULT res;
	LPCTSTR msg;
	HRECORDER hRec = (HRECORDER)malloc(sizeof(SoundRecorderTag));

	hRec->wf.wFormatTag = WAVE_FORMAT_PCM;
    hRec->wf.nChannels = 2;
    hRec->wf.nSamplesPerSec = 44100;
	hRec->wf.wBitsPerSample = 16;
	hRec->wf.nBlockAlign = hRec->wf.nChannels * hRec->wf.wBitsPerSample / 8;
    hRec->wf.nAvgBytesPerSec = hRec->wf.nSamplesPerSec * hRec->wf.nBlockAlign;
    hRec->wf.cbSize = 0;

	hRec->state = RS_STOPPED;

	hRec->nRecBytes = 0;
	
	lstrcpy(hRec->outFile, "out.mp3");

	res = waveInOpen(&hRec->hwi,
					 WAVE_MAPPER,
					 &hRec->wf,
					 (DWORD_PTR)(CallbackFuncRec),
					 0,
					 CALLBACK_FUNCTION);

	if (res != MMSYSERR_NOERROR)
	{
		switch (res)
		{
			case MMSYSERR_NOERROR : msg = "Peripherique d'entree ouvert"; break;
			case MMSYSERR_ALLOCATED : msg = "Deja alloue"; break;
			case MMSYSERR_BADDEVICEID  : msg = "Mauvais periph"; break;
 			case MMSYSERR_NOMEM  : msg = "Erreur de memoire"; break;
			case WAVERR_BADFORMAT  : msg = "Format non supporte"; break;
			case WAVERR_SYNC  : msg = "Erreur de synchro"; break;
			default : msg = "Erreur a la con";
		}
		fprintf(stderr, "%s\n", msg);
		free(hRec);
		return NULL;
	}

	return hRec;
}

void SoundRecorderClose(HRECORDER hRec)
{
	if (!hRec)
		return;
	SoundRecorderStop(hRec);
	waveInClose(hRec->hwi);
	free(hRec);
}

void CALLBACK CallbackFuncRec(HWAVEIN hwi, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2)
{
	HRECORDER hRec;
	WAVEHDR * hdr;
	DWORD nbWrtn;

	int nSamples;
	int nEncoded;

    if (uMsg == WIM_DATA)
    {
		hdr = (WAVEHDR *)(DWORD_PTR)dwParam1;
		hRec = (HRECORDER)hdr->dwUser;

		if (hdr->dwBytesRecorded > 0)
		{
			// encodage mp3
			nSamples = hdr->dwBytesRecorded / hRec->wf.nBlockAlign;

			nEncoded = lame_encode_buffer_interleaved(
				hRec->gfp,
				(short int*)hdr->lpData,
				nSamples,
				hRec->mp3_buffer,
				MP3_BUFF_SIZE
			);
			//

			// ecriture dans le fichier temporaire
			WriteFile(hRec->hTmpFile, hRec->mp3_buffer, nEncoded, &nbWrtn, NULL);
			//
			hRec->nRecBytes += hdr->dwBytesRecorded;
		}

		if (hRec->state == RS_RECORDING || hRec->state == RS_PAUSED)
		{
			waveInAddBuffer(hRec->hwi, hdr, sizeof(WAVEHDR));
		}
		else if (hRec->state == RS_STOPPED)
		{
			if (--hRec->usedBuffers == 0)
			{
				nEncoded =  lame_encode_flush(
					hRec->gfp,
					hRec->mp3_buffer,
					MP3_BUFF_SIZE
				);
				WriteFile(hRec->hTmpFile, hRec->mp3_buffer, nEncoded, &nbWrtn, NULL);
				lame_close(hRec->gfp);

				// fermeture du fichier
				CloseHandle(hRec->hTmpFile);
				//
			}
		}
    }
}

void SoundRecorderStart(HRECORDER hRec)
{
	int i;
	
	if (hRec->state == RS_STOPPED)
	{
		hRec->usedBuffers = 0;
		hRec->hdr = (WAVEHDR**)malloc(sizeof(WAVEHDR*) * NBUFFERS);

		//on lui fout n buffers et on les envoie
		for (i = 0; i < NBUFFERS; i++)
		{
			hRec->hdr[i] = (WAVEHDR*)malloc(sizeof(WAVEHDR));
			hRec->hdr[i]->lpData = (LPSTR)malloc(sizeof(CHAR) * hRec->wf.nAvgBytesPerSec);
			hRec->hdr[i]->dwBufferLength = hRec->wf.nAvgBytesPerSec;
			hRec->hdr[i]->dwFlags = 0;
			hRec->hdr[i]->dwBytesRecorded = 0;
			hRec->hdr[i]->dwUser = (DWORD_PTR)hRec;

			waveInPrepareHeader(hRec->hwi, hRec->hdr[i], sizeof(WAVEHDR));
			waveInAddBuffer(hRec->hwi, hRec->hdr[i], sizeof(WAVEHDR));

			++hRec->usedBuffers;
		}

		hRec->nRecBytes = 0;

		// cration du fichier temporaire
		hRec->hTmpFile = CreateFile(hRec->outFile, GENERIC_WRITE, 0, NULL,
			CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		//

		// init de l'encodeur mp3
		hRec->gfp = lame_init();

		lame_set_errorf(hRec->gfp,printlog);
		lame_set_debugf(hRec->gfp,printlog);
		lame_set_msgf(hRec->gfp,printlog);

		lame_set_num_channels(hRec->gfp,2);
		lame_set_in_samplerate(hRec->gfp,44100);
		lame_set_brate(hRec->gfp,128);
		lame_set_mode(hRec->gfp,JOINT_STEREO);
		lame_set_quality(hRec->gfp,5);

		id3tag_init(hRec->gfp);
		id3tag_v1_only(hRec->gfp);
		id3tag_set_comment(hRec->gfp, "Encoded with AniRecorder");

		lame_init_params(hRec->gfp);
		//

		waveInStart(hRec->hwi);
		hRec->state = RS_RECORDING;
	}
}

void SoundRecorderPause(HRECORDER hRec)
{
	if (hRec->state == RS_RECORDING)
	{
		hRec->state = RS_PAUSED;
		waveInStop(hRec->hwi);
	}
}

void SoundRecorderResume(HRECORDER hRec)
{
	if (hRec->state == RS_PAUSED)
	{
		hRec->state = RS_RECORDING;
		waveInStart(hRec->hwi);
	}
}

void SoundRecorderStop(HRECORDER hRec)
{
	int i;
	if (hRec->state == RS_RECORDING || hRec->state == RS_PAUSED)
	{
		hRec->state = RS_STOPPED;
		waveInReset(hRec->hwi);

		for (i = 0; i < 3; i++)
		{
			waveInUnprepareHeader(hRec->hwi, hRec->hdr[i], sizeof(WAVEHDR));
			free(hRec->hdr[i]->lpData);
			free(hRec->hdr[i]);
		}
		free(hRec->hdr);
	}
}

RecorderState SoundRecorderGetState(HRECORDER hRec)
{
	return hRec->state;
}

void SoundRecorderSetOutputFile(HRECORDER hRec, LPCTSTR file)
{
	lstrcpyn(hRec->outFile, file, MAX_PATH);
}

int SoundRecoderGetSecondsRecorded(HRECORDER hRec)
{
	return (int)((double)hRec->nRecBytes / (double)hRec->wf.nAvgBytesPerSec);
}
