#include <iostream.h>
#include <windows.h>
#include <mmsystem.h>
#include <mmreg.h>

bool rec_end;
bool play_end;

void getGSMFormat(PWAVEFORMATEX wfx)
{
	wfx->wFormatTag			= WAVE_FORMAT_GSM610;
	wfx->nChannels			= 1;
	wfx->nSamplesPerSec		= 8000;
	wfx->wBitsPerSample		= 0;
	wfx->nAvgBytesPerSec	= 1625;
	wfx->nBlockAlign		= 65;
	wfx->cbSize				= sizeof(WORD);
}

void CALLBACK CallbackFuncRec(HWAVEIN hwi,       
  UINT uMsg,         
  DWORD dwInstance,  
  DWORD dwParam1,    
  DWORD dwParam2)
{
	//appele quand le buffer d'enregistrement est vide
	if (uMsg == WIM_DATA)
	{
		cout << endl << "Le buffer est plein !" << endl;
		//waveInStop(hwi);
		//waveInReset(hwi);
		rec_end = true;
	}
	return;
}

void CALLBACK CallbackFuncPlay(HWAVEOUT hwo,       
  UINT uMsg,         
  DWORD dwInstance,  
  DWORD dwParam1,    
  DWORD dwParam2)
{
	//appele quand le buffer de lecture est vide
	if (uMsg == WOM_DONE)
	{
		cout << endl << "Le buffer est vide !" << endl;
		//waveOutReset(hwo);
		play_end = true;
	}
}


void main()
{
	MMRESULT res;				//resultat des foncs mmio
	LPGSM610WAVEFORMAT pgwf;	//format gsm
	HWAVEIN hrec;				//handle d'entree
	HWAVEOUT hplay;				//handle de sortie
	LPWAVEHDR lpwhdr;			//structure de contrle des buffers

	pgwf = new GSM610WAVEFORMAT;

	getGSMFormat(&pgwf->wfx);
		
	pgwf->wSamplesPerBlock = 320;

	cout << "Test des capacites audio GSM de Windows :" << endl
		 << endl
		 << "Format GSM" << endl
		 << "Canaux : " << pgwf->wfx.nChannels << endl
		 << "Echantillonnage : " << pgwf->wfx.nSamplesPerSec << " Hz" << endl
		 << "Bits par echantillon : " << pgwf->wfx.wBitsPerSample << endl
		 << "Bitrate : " << pgwf->wfx.nAvgBytesPerSec << " octets/sec" << endl
		 << "Echantillons par bloc : " << pgwf->wfx.nBlockAlign << endl
		 << endl;

/* On va enregistrer */

	/* Ouverture du priphrique WAVE_IN */
	res = waveInOpen( 
        &hrec,									// handle 
        WAVE_MAPPER,							// peripherique
        (LPWAVEFORMATEX)&(pgwf->wfx),           // format
        (DWORD)CallbackFuncRec,					// callback
        NULL,									// pas de donnes d'instance 
        CALLBACK_FUNCTION );

	switch (res)
	{
		case MMSYSERR_NOERROR : cout << "Peripherique d'entree ouvert"; break;
		case MMSYSERR_ALLOCATED : cout << "Deja alloue"; break;
		case MMSYSERR_BADDEVICEID  : cout << "Mauvais periph"; break;
		
 		case MMSYSERR_NOMEM  : cout << "Erreur de memoire"; break;
		case WAVERR_BADFORMAT  : cout << "Format non supporte"; break;
		case WAVERR_SYNC  : cout << "Erreur de synchro"; break;
		default : cout << "Erreur a la con";
	}
	cout << endl;
	if (res != MMSYSERR_NOERROR)
		return;					//quit si erreur

	lpwhdr = new WAVEHDR; //on alloue un wave header
	
	lpwhdr->lpData = new char[10000];
	lpwhdr->dwBufferLength = 10000;
	lpwhdr->dwFlags = 0;

	/* Prparation du header */
	res = waveInPrepareHeader(hrec, lpwhdr, sizeof(WAVEHDR));

	switch(res)
	{
		case MMSYSERR_NOERROR : cout << "Header prepare"; break;
		case MMSYSERR_NODRIVER  : cout << "Pas de driver"; break;
		case MMSYSERR_NOMEM  : cout << "Erreur de memoire"; break;
		case MMSYSERR_INVALHANDLE : cout << "Handle non valide"; break;
		default : cout << "Erreur a la con";
	}
	cout << endl;

	if (res != MMSYSERR_NOERROR)
	{
		waveInClose(hrec);
		return;
	}

	/* Ajouter le buffer */
	res = waveInAddBuffer(hrec, lpwhdr, sizeof(WAVEHDR));
	
	if (res != MMSYSERR_NOERROR)
	{
		waveInUnprepareHeader(hrec, lpwhdr, sizeof(WAVEHDR));
		waveInClose(hrec);
		return;
	}
	else
		cout << "Buffer ajoute" << endl;
	
	/* Reseter l'enregistrement */
	/*res = waveInReset(hrec);
	
	if (res != MMSYSERR_NOERROR)
	{
		waveInUnprepareHeader(hrec, lpwhdr, sizeof(WAVEHDR));
		waveInClose(hrec);
		return;
	}
	else*/
		cout << "Enregistrement initialise" << endl;

	/* Commencer l'enregistrement */
	rec_end = false;
	res = waveInStart(hrec);

	if (res != MMSYSERR_NOERROR)
	{
		waveInUnprepareHeader(hrec, lpwhdr, sizeof(WAVEHDR));
		waveInClose(hrec);
		return;
	}
	else
		cout << "Enregistrement en cours... (Parlez dans le MICRO !)" << endl;

	/* attendons la fin de l'enregistrement */
	while (!rec_end)
	{
		cout << "." << flush;
		Sleep(1000);
	}
	while (!(lpwhdr->dwFlags & WHDR_DONE));
	waveInReset(hrec);

	cout << "Fin de l'enregistrement" << endl;

	/* Librons le buffer ! */
	res = waveInUnprepareHeader(hrec, lpwhdr, sizeof(WAVEHDR));
	
	if (res != MMSYSERR_NOERROR)
	{
		waveInClose(hrec);
		return;
	}
	cout << "Buffer libere" << endl;

	/* On a enregistr notre son, on a le buffer, fermons l'enregistrement */
	waveInClose(hrec);

	cout << "Peripherique d'enregistrement ferme" << endl;

/* Maintenant on peut lire ce qu'on a enregistr */
	
	/* Prparons d'abord le buffer : ne pas comptabiliser les octets non enregistrs */
	lpwhdr->dwBufferLength = lpwhdr->dwBytesRecorded;
	lpwhdr->dwFlags = 0;

	/* On ouvre le priph de sortie */
	res = waveOutOpen( 
        &hplay,									// handle 
        WAVE_MAPPER,							// peripherique
        (LPWAVEFORMATEX)&(pgwf->wfx),           // format
        (DWORD)CallbackFuncPlay,				// callback
        NULL,									// pas de donnes d'instance 
        CALLBACK_FUNCTION );
	
	if (res != MMSYSERR_NOERROR)
	{
		return;
	}
	cout << "Peripherique de sortie ouvert" << endl;

	/* On prpare le buffer de sortie */
	res = waveOutPrepareHeader(hplay, lpwhdr, sizeof(WAVEHDR));
	if (res != MMSYSERR_NOERROR)
	{
		waveOutClose(hplay);
		return;
	}
	cout << "Header prepare" << endl;

	/* On lit le buffer */
	play_end = false;
	res = waveOutWrite(hplay, lpwhdr, sizeof(WAVEHDR));
	if (res != MMSYSERR_NOERROR)
	{
		waveOutUnprepareHeader(hplay, lpwhdr, sizeof(WAVEHDR));
		waveOutClose(hplay);
		return;
	}
	cout << "Lecture en cours" << endl;

	/* Attendons la fin */
	while(!play_end)
	{
		cout << "." << flush;
		Sleep(1000);
	}
	while (!(lpwhdr->dwFlags & WHDR_DONE));
	waveOutReset(hplay);

	cout << "Fin de la lecture" << endl;

	/* Librons le buffer ! */
	res = waveOutUnprepareHeader(hplay, lpwhdr, sizeof(WAVEHDR));
	if (res != MMSYSERR_NOERROR)
	{
		waveOutClose(hplay);
		return;
	}
	cout << "Buffer libere" << endl;

	/* Fermons le priphrique de sortie */
	waveOutClose(hplay);
	cout << "Peripherique de sortie ferme" << endl;
	
	/*Librons la mmoire*/
	delete [] lpwhdr->lpData;
	delete lpwhdr;
	delete pgwf;
}
